#include "ble_cnt_service.h"
#include <string.h>
#include "nordic_common.h"
#include "ble_srv_common.h"
#include "app_util.h"

static void on_connect(ble_cnts_t * p_cnts, ble_evt_t * p_ble_evt)
{
    p_cnts->conn_handle = p_ble_evt->evt.gap_evt.conn_handle;
}

static void on_disconnect(ble_cnts_t * p_cnts, ble_evt_t * p_ble_evt)
{
    UNUSED_PARAMETER(p_ble_evt);
    p_cnts->conn_handle = BLE_CONN_HANDLE_INVALID;
}


static void on_write(ble_cnts_t * p_cnts, ble_evt_t * p_ble_evt)
{
    ble_gatts_evt_write_t * p_evt_write = &p_ble_evt->evt.gatts_evt.params.write;

   if ((p_evt_write->handle == p_cnts->cnts_handles.value_handle) &&
       (p_evt_write->len == 1) &&
       (p_cnts->cnts_write_handler != NULL))
   {
	   p_cnts->cnts_write_handler(p_cnts, p_evt_write->data[0]);
   }
}


void ble_cnts_on_ble_evt(ble_cnts_t * p_cnts, ble_evt_t * p_ble_evt)
{
    switch (p_ble_evt->header.evt_id)
    {
        case BLE_GAP_EVT_CONNECTED:
            on_connect(p_cnts, p_ble_evt);
            break;

        case BLE_GAP_EVT_DISCONNECTED:
            on_disconnect(p_cnts, p_ble_evt);
            break;

        case BLE_GATTS_EVT_WRITE:
            on_write(p_cnts, p_ble_evt);
            break;
        default:
            break;
    }
}

static uint32_t cnts_char_add(ble_cnts_t * p_cnts, const ble_cnts_init_t * p_cnts_init)
{
    ble_gatts_char_md_t char_md;
    ble_gatts_attr_md_t cccd_md;
    ble_gatts_attr_t    attr_char_value;
    ble_uuid_t          ble_uuid;
    ble_gatts_attr_md_t attr_md;


    memset(&cccd_md, 0, sizeof(cccd_md));

    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&cccd_md.read_perm);
    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&cccd_md.write_perm);

    cccd_md.vloc = BLE_GATTS_VLOC_STACK;

    memset(&char_md, 0, sizeof(char_md));

    char_md.char_props.read   = 1;
    char_md.char_props.notify = 1;
    char_md.p_char_user_desc  = NULL;
    char_md.p_char_pf         = NULL;
    char_md.p_user_desc_md    = NULL;
    char_md.p_cccd_md         = &cccd_md;
    char_md.p_sccd_md         = NULL;

    ble_uuid.type = p_cnts->uuid_type;
    ble_uuid.uuid =  CNTS_UUID_COUNT;

    memset(&attr_md, 0, sizeof(attr_md));

    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&attr_md.read_perm);
    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&attr_md.write_perm);

    attr_md.vloc       = BLE_GATTS_VLOC_STACK;
    attr_md.rd_auth    = 0;
    attr_md.wr_auth    = 0;
    attr_md.vlen       = 0;

    memset(&attr_char_value, 0, sizeof(attr_char_value));

    attr_char_value.p_uuid       = &ble_uuid;
    attr_char_value.p_attr_md    = &attr_md;
    attr_char_value.init_len     = sizeof(uint8_t);
    attr_char_value.init_offs    = 0;
    attr_char_value.max_len      = sizeof(uint8_t);
    attr_char_value.p_value      = NULL;

    return sd_ble_gatts_characteristic_add(p_cnts->service_handle, &char_md,
                                               &attr_char_value,
                                               &p_cnts->cnts_handles);
}


uint32_t ble_cnts_init(ble_cnts_t * p_cnts, const ble_cnts_init_t * p_lbs_init)
{
    uint32_t   err_code;
    ble_uuid_t ble_uuid;

    // Initialize service structure
    p_cnts->conn_handle       = BLE_CONN_HANDLE_INVALID;
   // p_cnts->led_write_handler = p_lbs_init->cnts_write_handler;

    // Add base UUID to softdevice's internal list.
    ble_uuid128_t base_uuid = CNTS_UUID_BASE;
    err_code = sd_ble_uuid_vs_add(&base_uuid, &p_cnts->uuid_type);
    if (err_code != NRF_SUCCESS)
    {
        return err_code;
    }

    ble_uuid.type = p_cnts->uuid_type;
    ble_uuid.uuid = CNTS_UUID_SERVICE;

    err_code = sd_ble_gatts_service_add(BLE_GATTS_SRVC_TYPE_PRIMARY, &ble_uuid, &p_cnts->service_handle);
    if (err_code != NRF_SUCCESS)
    {
        return err_code;
    }

    err_code = cnts_char_add(p_cnts, p_lbs_init);
    if (err_code != NRF_SUCCESS)
    {
        return err_code;
    }

    //err_code  = led_char_add(p_lbs, p_lbs_init);
    //if (err_code != NRF_SUCCESS)
    //{
    //    return err_code;
    //}

    return NRF_SUCCESS;
}

uint32_t ble_cnts_change(ble_cnts_t * p_cnts, uint8_t count)
{
    ble_gatts_hvx_params_t params;
    uint16_t len = sizeof(count);
    uint32_t err_code = NRF_SUCCESS;

	memset(&params, 0, sizeof(params));
	params.type = BLE_GATT_HVX_NOTIFICATION;
	params.handle = p_cnts->cnts_handles.value_handle;
	params.p_data = &count;
	params.p_len = &len;

	return sd_ble_gatts_hvx(p_cnts->conn_handle, &params);

    return 0;
}


